import { fromEvent } from 'rxjs/observable/fromEvent';
import { filter, map, takeUntil, withLatestFrom } from 'rxjs/operators';
var Key;
(function (Key) {
    Key[Key["Tab"] = 9] = "Tab";
})(Key || (Key = {}));
var FOCUSABLE_ELEMENTS_SELECTOR = [
    'a[href]', 'button:not([disabled])', 'input:not([disabled]):not([type="hidden"])', 'select:not([disabled])',
    'textarea:not([disabled])', '[contenteditable]', '[tabindex]:not([tabindex="-1"])'
].join(', ');

var currentTab = 0;
/**
 * Returns first and last focusable elements inside of a given element based on specific CSS selector
 */
function getFocusableBoundaryElements(element) {
    var list = element.querySelectorAll(FOCUSABLE_ELEMENTS_SELECTOR);
    var listTemp = [];
    for (let i = 0; i < list.length; i++) {
        if (list[i].className !== 'mat-icon-button' &&
            !list[i].disabled) {
            listTemp.push(list[i]);
        }
    }

    console.log('getFocusableBoundaryElements', listTemp);
    return [listTemp[0], listTemp[1], listTemp[listTemp.length - 1]];
    // return listTemp;
}
/**
 * Function that enforces browser focus to be trapped inside a DOM element.
 *
 * Works only for clicks inside the element and navigation with 'Tab', ignoring clicks outside of the element
 *
 * @param element The element around which focus will be trapped inside
 * @param stopFocusTrap$ The observable stream. When completed the focus trap will clean up listeners
 * and free internal resources
 */
export var ngbFocusTrap = function (element, stopFocusTrap$) {
    // last focused element
    var lastFocusedElement$ = fromEvent(element, 'focusin').pipe(takeUntil(stopFocusTrap$), map(function (e) { return e.target; }));
    // 'tab' / 'shift+tab' stream
    fromEvent(element, 'keydown')
        .pipe(takeUntil(stopFocusTrap$), filter(function (e) { return e.which === Key.Tab; }), withLatestFrom(lastFocusedElement$))
        .subscribe(function (_a) {
            var tabEvent = _a[0], focusedElement = _a[1];
            var _b = getFocusableBoundaryElements(element), first = _b[0], last = _b[_b.length - 1];
            // if (focusedElement === first) {
            //     _b[1].focus();
            //     currentTab = 1;
            //     tabEvent.preventDefault();
            // } else if (focusedElement === last) {
            //     _b[0].focus();
            //     currentTab = 0;
            //     tabEvent.preventDefault();
            // }

            var _b = getFocusableBoundaryElements(element), first = _b[0], last = _b[2], middle = _b[1];
            console.log('_b nè', _b);
            console.log('_a nè', _a);
            console.log('focusedElement', focusedElement);
            console.log('lastlastlast', last);
            if (focusedElement === first && tabEvent.shiftKey) {
                console.log('first focus()');
                last.focus();
                tabEvent.preventDefault();
            }

            if (focusedElement === first && !tabEvent.shiftKey) {
                console.log('middle focus()');
                middle.focus();
                tabEvent.preventDefault();
            }

            if (focusedElement === last && !tabEvent.shiftKey) {
                console.log('last focus()');
                first.focus();
                tabEvent.preventDefault();
            }
        });
    // inside click
    fromEvent(element, 'click')
        .pipe(takeUntil(stopFocusTrap$), withLatestFrom(lastFocusedElement$), map(function (arr) { return arr[1]; }))
        .subscribe(function (lastFocusedElement) { return lastFocusedElement.focus(); });
};
//# sourceMappingURL=focus-trap.js.map